@@ -15,7 +15,6 @@ from fnmatch import fnmatchcase
 from subprocess import list2cmdline
 from threading import Thread
 
-import importlib_metadata
 import pluggy
 import py
 import toml
@@ -34,8 +33,10 @@ from tox.reporter import (
     verbosity1,
 )
 from tox.util.path import ensure_empty_dir
+from tox.util.stdlib import importlib_metadata
 
-from .parallel import ENV_VAR_KEY as PARALLEL_ENV_VAR_KEY
+from .parallel import ENV_VAR_KEY_PRIVATE as PARALLEL_ENV_VAR_KEY_PRIVATE
+from .parallel import ENV_VAR_KEY_PUBLIC as PARALLEL_ENV_VAR_KEY_PUBLIC
 from .parallel import add_parallel_config, add_parallel_flags
 from .reporter import add_verbosity_commands
 
@@ -51,11 +52,11 @@ hookimpl = tox.hookimpl
 Import hookimpl directly from tox instead.
 """
 
-default_factors = tox.PYTHON.DEFAULT_FACTORS
-"""DEPRECATED MOVE - please update to new location."""
-
 WITHIN_PROVISION = os.environ.get(str("TOX_PROVISION")) == "1"
 
+INTERRUPT_TIMEOUT = 0.3
+TERMINATE_TIMEOUT = 0.2
+
 
 def get_plugin_manager(plugins=()):
     # initialize plugin manager
@@ -549,11 +550,21 @@ def tox_addoption(parser):
         python conflict is set in which case the factor name implied version if forced
         """
         for factor in testenv_config.factors:
-            if factor in tox.PYTHON.DEFAULT_FACTORS:
-                implied_python = tox.PYTHON.DEFAULT_FACTORS[factor]
+            match = tox.PYTHON.PY_FACTORS_RE.match(factor)
+            if match:
+                base_exe = {"py": "python"}.get(match.group(1), match.group(1))
+                version_s = match.group(2)
+                if not version_s:
+                    version_info = ()
+                elif len(version_s) == 1:
+                    version_info = (version_s,)
+                else:
+                    version_info = (version_s[0], version_s[1:])
+                implied_version = ".".join(version_info)
+                implied_python = "{}{}".format(base_exe, implied_version)
                 break
         else:
-            implied_python, factor = None, None
+            implied_python, version_info, implied_version = None, (), ""
 
         if testenv_config.config.ignore_basepython_conflict and implied_python is not None:
             return implied_python
@@ -561,23 +572,20 @@ def tox_addoption(parser):
         proposed_python = (implied_python or sys.executable) if value is None else str(value)
         if implied_python is not None and implied_python != proposed_python:
             testenv_config.basepython = proposed_python
-            match = tox.PYTHON.PY_FACTORS_RE.match(factor)
-            implied_version = match.group(2) if match else None
-            if implied_version is not None:
-                python_info_for_proposed = testenv_config.python_info
-                if not isinstance(python_info_for_proposed, NoInterpreterInfo):
-                    proposed_version = "".join(
-                        str(i) for i in python_info_for_proposed.version_info[0:2]
-                    )
-                    # '27'.startswith('2') or '27'.startswith('27')
-                    if not proposed_version.startswith(implied_version):
-                        # TODO(stephenfin): Raise an exception here in tox 4.0
-                        warnings.warn(
-                            "conflicting basepython version (set {}, should be {}) for env '{}';"
-                            "resolve conflict or set ignore_basepython_conflict".format(
-                                proposed_version, implied_version, testenv_config.envname
-                            )
+            python_info_for_proposed = testenv_config.python_info
+            if not isinstance(python_info_for_proposed, NoInterpreterInfo):
+                proposed_version = ".".join(
+                    str(x) for x in python_info_for_proposed.version_info[: len(version_info)]
+                )
+                if proposed_version != implied_version:
+                    # TODO(stephenfin): Raise an exception here in tox 4.0
+                    warnings.warn(
+                        "conflicting basepython version (set {}, should be {}) for env '{}';"
+                        "resolve conflict or set ignore_basepython_conflict".format(
+                            proposed_version, implied_version, testenv_config.envname
                         )
+                    )
+
         return proposed_python
 
     parser.add_testenv_attribute(
@@ -659,14 +667,20 @@ def tox_addoption(parser):
         value = list(itertools.chain.from_iterable([x.split(" ") for x in value]))
 
         passenv = {
-            "PATH",
-            "PIP_INDEX_URL",
+            "CURL_CA_BUNDLE",
             "LANG",
             "LANGUAGE",
             "LD_LIBRARY_PATH",
+            "PATH",
+            "PIP_INDEX_URL",
+            "REQUESTS_CA_BUNDLE",
+            "SSL_CERT_FILE",
             "TOX_WORK_DIR",
+            "HTTP_PROXY",
+            "HTTPS_PROXY",
+            "NO_PROXY",
             str(REPORTER_TIMESTAMP_ON_ENV),
-            str(PARALLEL_ENV_VAR_KEY),
+            str(PARALLEL_ENV_VAR_KEY_PUBLIC),
         }
 
         # read in global passenv settings
@@ -790,6 +804,20 @@ def tox_addoption(parser):
 
     parser.add_testenv_attribute_obj(DepOption())
 
+    parser.add_testenv_attribute(
+        name="interrupt_timeout",
+        type="float",
+        default=INTERRUPT_TIMEOUT,
+        help="timeout before sending SIGTERM after SIGINT",
+    )
+
+    parser.add_testenv_attribute(
+        name="terminate_timeout",
+        type="float",
+        default=TERMINATE_TIMEOUT,
+        help="timeout before sending SIGKILL after SIGTERM",
+    )
+
     parser.add_testenv_attribute(
         name="commands",
         type="argvlist",
@@ -983,11 +1011,12 @@ class ParseIni(object):
         self.config = config
 
         prefix = "tox" if ini_path.basename == "setup.cfg" else None
+        fallbacksection = "tox:tox" if ini_path.basename == "setup.cfg" else "tox"
 
         context_name = getcontextname()
         if context_name == "jenkins":
             reader = SectionReader(
-                "tox:jenkins", self._cfg, prefix=prefix, fallbacksections=["tox"]
+                "tox:jenkins", self._cfg, prefix=prefix, fallbacksections=[fallbacksection]
             )
             dist_share_default = "{toxworkdir}/distshare"
         elif not context_name:
@@ -1026,7 +1055,7 @@ class ParseIni(object):
         config.sdistsrc = reader.getpath("sdistsrc", None)
         config.setupdir = reader.getpath("setupdir", "{toxinidir}")
         config.logdir = config.toxworkdir.join("log")
-        within_parallel = PARALLEL_ENV_VAR_KEY in os.environ
+        within_parallel = PARALLEL_ENV_VAR_KEY_PRIVATE in os.environ
         if not within_parallel and not WITHIN_PROVISION:
             ensure_empty_dir(config.logdir)
 
@@ -1161,6 +1190,9 @@ class ParseIni(object):
             # noinspection PyBroadException
             try:
                 package = requirements.Requirement(require)
+                # check if the package even applies
+                if package.marker and not package.marker.evaluate({"extra": ""}):
+                    continue
                 package_name = canonicalize_name(package.name)
                 if package_name not in exists:
                     deps.append(DepConfig(require, None))
@@ -1219,7 +1251,16 @@ class ParseIni(object):
         for env_attr in config._testenv_attr:
             atype = env_attr.type
             try:
-                if atype in ("bool", "path", "string", "dict", "dict_setenv", "argv", "argvlist"):
+                if atype in (
+                    "bool",
+                    "float",
+                    "path",
+                    "string",
+                    "dict",
+                    "dict_setenv",
+                    "argv",
+                    "argvlist",
+                ):
                     meth = getattr(reader, "get{}".format(atype))
                     res = meth(env_attr.name, env_attr.default, replace=replace)
                 elif atype == "basepython":
@@ -1275,7 +1316,7 @@ class ParseIni(object):
             all_envs = self._getallenvs(reader)
         else:
             candidates = (
-                (os.environ.get(PARALLEL_ENV_VAR_KEY), True),
+                (os.environ.get(PARALLEL_ENV_VAR_KEY_PRIVATE), True),
                 (from_option, True),
                 (from_environ, True),
                 ("py" if self.config.option.devenv is not None else None, False),
@@ -1436,6 +1477,20 @@ class SectionReader:
 
         return d
 
+    def getfloat(self, name, default=None, replace=True):
+        s = self.getstring(name, default, replace=replace)
+        if not s or not replace:
+            s = default
+        if s is None:
+            raise KeyError("no config value [{}] {} found".format(self.section_name, name))
+
+        if not isinstance(s, float):
+            try:
+                s = float(s)
+            except ValueError:
+                raise tox.exception.ConfigError("{}: invalid float {!r}".format(name, s))
+        return s
+
     def getbool(self, name, default=None, replace=True):
         s = self.getstring(name, default, replace=replace)
         if not s or not replace:
